@@ -2,11 +2,13 @@
 
 /**
   * SquirrelMail S/MIME Verification Plugin
-  * Copyright (c) 2001-2003 Wouter Teepe <wouter@teepe.com>,
-  * Copyright (c) 2003 Antonio Vasconcelos <vasco@threatconn.com>,
-  * Copyright (c) 2004 Scott Heavner,
-  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
-  * Copyright (c) 2005 Khedron Wilk <khedron@wilk.se>,
+  *
+  * Copyright (c) 2005-2012 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2005 Khedron Wilk <khedron@wilk.se>
+  * Copyright (c) 2004 Scott Heavner
+  * Copyright (c) 2003 Antonio Vasconcelos <vasco@threatconn.com>
+  * Copyright (c) 2001-2003 Wouter Teepe <wouter@teepe.com>
+  *
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -64,6 +66,7 @@ if (isset($cert))
       SendDownloadHeaders('application', 'octet-stream', 'cert.pem', 1);
    else
       DumpHeaders('application', 'octet-stream', 'cert.pem', 1);
+   if (substr($cert_in_dir, -1) !== '/') $cert_in_dir .= '/';
    $fd = fopen($cert_in_dir . $cert, 'r');
    while (!feof($fd))
    {
@@ -90,8 +93,8 @@ if (isset($cert))
         $isIE = 1;
       }
       
-      $filename = ereg_replace('[^-a-zA-Z0-9\.]', '_', $filename);
-      
+      $filename = preg_replace('/[^-a-zA-Z0-9.]/', '_', $filename);
+
       // A Pox on Microsoft and it's Office!
       if (! $force)
       {
