@@ -2,11 +2,13 @@
 
 /**
   * SquirrelMail S/MIME Verification Plugin
-  * Copyright (c) 2001-2003 Wouter Teepe <wouter@teepe.com>,
-  * Copyright (c) 2003 Antonio Vasconcelos <vasco@threatconn.com>,
-  * Copyright (c) 2004 Scott Heavner,
-  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
-  * Copyright (c) 2005 Khedron Wilk <khedron@wilk.se>,
+  *
+  * Copyright (c) 2005-2012 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2005 Khedron Wilk <khedron@wilk.se>
+  * Copyright (c) 2004 Scott Heavner
+  * Copyright (c) 2003 Antonio Vasconcelos <vasco@threatconn.com>
+  * Copyright (c) 2001-2003 Wouter Teepe <wouter@teepe.com>
+  *
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -132,16 +134,18 @@ function smime_check_configuration_do()
   * Verify the signature on a signed message
   *
   * @param string The full message being verified
+  * @param string The sender/from address for the message if available
   *
   * @return array A list of the OpenSSL verification result value,
   *               the full command output, the name of the sender,
   *               and the certificate
   *
   */
-function verify_smime($message_in)
+function verify_smime($message_in, $sender_address='')
 {
 
    global $openssl, $echo, $cadir, $easycerts, $message, $cert_in_dir;
+   if (substr($cert_in_dir, -1) !== '/') $cert_in_dir .= '/';
 
    smime_init();
 
@@ -159,7 +163,9 @@ function verify_smime($message_in)
    {
       passthru("$echo $subjectmessage | $openssl smime -verify -CApath $cadir $easycerts 2> /dev/null >/dev/null", $retval);
       if ($retval == 4) $retval = 6;
-      exec("$openssl x509 -in $tmpcert -subject -hash -noout", $lines);
+      // OLD: why didn't the original author use the -email option?
+      // $hash = exec("$openssl x509 -in $tmpcert -subject -hash -noout", $lines);
+      $hash = exec("$openssl x509 -in $tmpcert -subject -email -hash -noout", $lines);
 
 
 # [vasco]
@@ -175,10 +181,24 @@ function verify_smime($message_in)
 # [wouter]
 # This is something of an OR ...
 
-      preg_match("/\/CN=(.*)\/(emailAddress|Email)=(.*)(\/)?/", $lines[0], $res);
+      // OLD: if not using the -email option, we tried to extract the email ourselves:
+      //preg_match("/\/CN=(.*)\/(emailAddress|Email)=(.*)(\/)?/", $lines[0], $res);
 
-      $name = htmlentities($res[1] . ' <' . $res[3] . '>');
-      $longcert = $cert_in_dir . $lines[1] . '.O';
+      // get name
+      preg_match("/\/CN=(.+?)(\/|$)/", $lines[0], $matches);
+
+      // get email - use sender's address if it's in the certificate,
+      // otherwise just take the first one we can find
+      if (in_array($sender_address, $lines))
+         $email = $sender_address;
+      else if (!empty($lines[1]))
+         $email = $lines[1];
+      else
+         $email = '';
+
+      if (empty($matches[1])) $matches[1] = '';
+      $name = htmlentities($matches[1] . ' <' . $email . '>');
+      $longcert = $cert_in_dir . $hash . '.O';
       if (file_exists($longcert))
       {
          if (filesize($longcert) == filesize($tmpcert))
@@ -198,7 +218,7 @@ function verify_smime($message_in)
          }
          if (isset($unequal))
          {
-            $longcert = tempnam($cert_in_dir, $lines[1] . '.O');
+            $longcert = tempnam($cert_in_dir, $hash . '.O');
          }
          else
          {
@@ -281,15 +301,15 @@ function mime_fetch_full_body ($imap_stream, $id)
    $data = sqimap_run_command($imap_stream, $cmd, true, $response, $message, $uid_support);
    $topline = array_shift($data);
 
-   while (! ereg('\\* [0-9]+ FETCH ', $topline) && $data)
+   while (! preg_match('/\\* [0-9]+ FETCH /', $topline) && $data)
       $topline = array_shift($data);
 
    $wholemessage = implode('', $data);
-   if (ereg('\\{([^\\}]*)\\}', $topline, $regs))
+   if (preg_match('/\\{([^\\}]*)\\}/', $topline, $regs))
    {
       return substr($wholemessage, 0, $regs[1]);
    }
-   else if (ereg('"([^"]*)"', $topline, $regs))
+   else if (preg_match('/"([^"]*)"/', $topline, $regs))
    {
       return $regs[1];
    }
@@ -380,6 +400,16 @@ function smime_header_verify_do()
 //      $passed_id = $passed_ent_id;
 
 
+   // grab the sender address
+   // (AddressStructure class stupidly has no way to get just the email address)
+   $sender_address = '';
+   if (!empty($message->rfc822_header->from[0]->mailbox))
+      $sender_address = (!empty($message->rfc822_header->from[0]->host)
+                      ? $message->rfc822_header->from[0]->mailbox
+                        . '@' . $message->rfc822_header->from[0]->host
+                      : $message->rfc822_header->from[0]->mailbox);
+
+
    if ($message->header->type0 == 'application' and $message->header->type1 == 'pkcs7-mime')
    {
 
@@ -429,7 +459,7 @@ function smime_header_verify_do()
       $cmd = "FETCH $passed_id BODY.PEEK[HEADER.FIELDS (Content-Type)]";
       $read = sqimap_run_command($imapConnection, $cmd, true, $response, $mess, $uid_support);
 
-      if (eregi('protocol=(")?application/(x-)?pkcs7-signature(")?', implode('', $read)))
+      if (preg_match('/protocol=(")?application\/(x-)?pkcs7-signature(")?/i', implode('', $read)))
       {
 
          // we have a detatched s/mime message
@@ -480,7 +510,7 @@ function smime_header_verify_do()
 */
 
          $body = mime_fetch_full_body ($imapConnection, $passed_id);
-         list ($retval, $lines, $name, $cert) = verify_smime($body);
+         list ($retval, $lines, $name, $cert) = verify_smime($body, $sender_address);
          $verify_status = convert_verify_result_to_displayable_text($retval);
 
 
