@@ -168,6 +168,7 @@ function x509_split($a, $b, $c)
 function x509_open($cert)
 {
    global $cert_in_dir, $openssl;
+   if (substr($cert_in_dir, -1) !== '/') $cert_in_dir .= '/';
 
    $lines = array();
    $safe_cert = escapeshellarg("$cert_in_dir$cert");
@@ -177,7 +178,7 @@ function x509_open($cert)
       x509_split(x509_match($lines[1], "issuer="), "", "\n"),
       x509_match($lines[2], "notBefore="),
       x509_match($lines[3], "notAfter="),
-      substr(ereg_replace("(..)", ":\\1", x509_match($lines[4], "serial=")),1), x509_match($lines[5], "MD5 Fingerprint=")
+      substr(preg_replace('/(..)/', ':$1', x509_match($lines[4], "serial=")),1), x509_match($lines[5], "MD5 Fingerprint=")
    );
 }
 
